/******************
Austin Moss
Automated Foam Defense Turret
11/10/18
*******************/

// Definitions

// Ouput Pins
#define MotorPin 5   //Define the pin used for the flywheel motors
#define PusherPin 6   // Define the pin to be used for the pusher motor which fires the darts.
#define LEDPin 11  // Define the pin for the flashing LED warning lights.
#define BuzzerPin 12  // Define the pin for the warning buzzer

// Input Pins
#include "HCSR04.h"  //include the proximity sensor library

#define echoPin 8    // This is the echo pin
#define triggerPin 7   // This is the trigger pin

#define interruptNumber 0   //interrupt 0 because we are using pin 2. 
#define TouchPadPin 2		//connect touch sensor output (pin "S") to D2 (and pin "G" to GND and pin "V" to 5V)
#define LED 13                  //define the input for the ON/OFF green LED

volatile byte TouchPadEventFlag;//all variables that are changed within an ISR need to be dimensioned as 'volatile'
                                //this variable will be used to tell the main loop that an interrupt event occurred.

HCSR04ProxSensor distanceSensor(echoPin,triggerPin);

// Setup 

void setup() {
  Serial.begin(9600);
  pinMode(LED, OUTPUT);//define the built in LED pin as output
  pinMode(TouchPadPin, INPUT);//define the interrupt pin as an input
  attachInterrupt(interruptNumber,InterruptServiceRoutine, CHANGE);//CHANGE defines that the ISR is called when the voltage on the pin changes in either direction
                                                                   //we are using CHANGE and not LOW to 'debounce' the pad. Using LOW would result in a continuous reading of the touch event until your finger
                                                                   //is removed from the pad
  pinMode(MotorPin, OUTPUT);  // Defines the flywheels as an output
  pinMode(PusherPin, OUTPUT);  // Defines the pusher motor as an output
  pinMode(LEDPin, OUTPUT);  // Defines the warning LEDs as an output
  pinMode(BuzzerPin, OUTPUT);  // Defines the warning buzzer as an output
  
}

// Main Code 

void loop() {
  distanceSensor.readSensor();
  distanceSensor.readSensor();  
  distanceSensor.readSensor();
  while (TouchPadEventFlag==1) //the loop for the system while it is in its "ON" state
  {
      float distance = distanceSensor.readSensor(); //Reading the proximity sensor
      Serial.println(distance);  //A system read of the distance. 
      Serial.println(distanceSensor.getLastValue()); // A system read of the last distance. 
      if (distanceSensor.getLastValue() - distance > 10 || distanceSensor.getLastValue() - distance < -10) // This is the check that activates the system, given a change of 10 cm.
      {                                                                                                    // If an object approaches the turret, or a door opens/ closes that the sensor is
                                                                                                           //checking, the "fire mode" will activate.  
          Serial.println("INTRUDER DETECTED");  //Notification that the sensor has been tripped.                                                
          digitalWrite(MotorPin,HIGH);  //Begin activating the flywheels
          warning();  // Begin the warning cycle.
          fireDarts();  // Enact defense protocols
          digitalWrite(MotorPin,LOW); // Turns off the flywheels now that intruder has been dealt with
          delay(2000); // Delay the system by 2 secs before begining guard.
          TouchPadEventFlag=0;   //Sets the system to off to reload  
          digitalWrite(LED, LOW);
      }
      delay(500); // .5 sec delay in between checks.
    }
  
  
}
